<TeXmacs|0.3.4.3>

<style|article>

<\body>
  <assign|code|<macro|body|<surround|<vspace*|1fn>|<vspace|1fn><format|no
  indentation after>|<with|font family|tt|language|verbatim|<arg|body>>>>><se\
  ction|Backup>

  <subsection|Multiple declaration parts in blocks?>

  It is also possible to put additinal declaration and instruction parts
  after the instruction part of the block:

  <\code>
    {

    \ \ <with|font shape|italic|decls>;

    \ \ <with|font shape|italic|insts>;

    \ \ <with|font shape|italic|decls>;

    \ \ <with|font shape|italic|insts>;

    }
  </code>

  being equivalent to

  <\code>
    {

    \ \ <with|font shape|italic|decls>;

    \ \ <with|font shape|italic|insts>;

    \ \ {

    \ \ \ \ <with|font shape|italic|decls>;

    \ \ \ \ <with|font shape|italic|insts>;

    \ \ }

    }
  </code>

  <subsection|Class specialization>

  Parameterized classes like polynomials are usually written in a generic way
  in Mathemagix. However, for a very elementary class like <with|font
  family|tt|Int>, it may be worth it to have more efficient specialized code
  for <with|font family|tt|Polynomial(Int)>. In Mathemagix, you may create a
  class <with|font family|tt|Polynomial_Int> for this purpose, which
  <with|font shape|italic|duplicates> and specializes the routines from the
  generic class:

  <\code>
    Polynomial_Int :: Ring

    with duplicate Polynomial(Int);
  </code>

  The <with|font family|tt|duplicate> keyword is used in a similar way as the
  <with|font family|tt|import> and <with|font family|tt|inherit> keywords,
  which makes it possible to duplicate only certain routines from <with|font
  family|tt|Polynomial(Int)> and inherit the rest:

  <\code>
    Polynomial_Int :: Ring

    with inherit * from Polynomial(Int);

    with duplicate except * from Polynomial(int);
  </code>

  It is not yet clear to us what the precise applicability conditions for the
  <with|font family|tt|duplicate> keyword should be. In its broadest
  interpretation, the construct should act as a macro, so that the code has
  to be recompiled completely. In narrower interpretations, we might wish to
  directly transpose the type checked code to the more specialized class. In
  this case both classes come to differ only at the optimization and the code
  generation phases.

  Conditional exports form a completely different approach for class
  specialization. However, the precise semantics of the conditions still has
  to be determined. Furthermore, conflicting specializations should be
  studied in greater detail.

  <subsection|Conditionally variable exports><label|disc-cv>

  Consider the following piece of code:

  <\code>
    R:$Ring := Integer;

    import R, Complex(R);

    x:R := 5;

    (x,R) := (x*I,Complex(R));
  </code>

  One might condier that this should be correct code, although our criterion
  that types should be constants when instanciated is not satisfied.

  Actually, this criterion may be refined, by introducing <with|font
  family|tt|depend[<with|font shape|italic|var>,...,<with|font
  shape|italic|var>]> modifiers in the implementation. These modifiers
  specify that an export is variable but that it depends on a sequence of
  other exports. The idea is as follows: assume that a <with|font
  family|tt|variable> class <with|font family|tt|C> is defined, with
  instances <with|font family|tt|var_1, ..., var_n> in the same scope. Then
  <with|font family|tt|C> will actually be <with|font
  family|tt|depend[var_1,...,var_n]> instead of <with|font
  family|tt|variable>. Consequently, <with|font family|tt|C> can only be
  modified if <with|font family|tt|var_1, <format|no line break>..., var_n>
  are modified accordingly at the same time.

  Notice however, that <with|font family|tt|depend> does not always behave
  like a real modifier, because new instances of <with|font family|tt|C>
  might be introduced in subblocks:

  <\code>
    C:$Ring := Integer;

    import C;

    x:C := 5;

    if <with|font shape|italic|cond> then {

    \ \ y:C := 8;

    \ \ ...

    }
  </code>

  One solution to this problem might be aliasing: in the inner block, the
  compiler implicitly creates a new variable <with|font family|tt|depend[x,y]
  C:$Ring>, which is assigned with the value of <with|font family|tt|C> from
  the outer block.

  <subsection|Project files>

  A project file may contain the following keywords:

  <\description>
    <expand|item*|Parent.>For indicating that the project has one or more
    parent projects.

    <expand|item*|Child.>For indicating that the project has one or more
    children.

    <expand|item*|Prefix.>A prefix for avoiding name clashes. All global
    symbols in the generated object files will be prefixed by this prefix.

    <expand|item*|Version.>The current version for optional version control.

    <expand|item*|Source.>For declaring a source file of the project and the
    header files on which it depends.

    <expand|item*|Header.>For declaring a header file of the project and its
    dependencies.

    <expand|item*|Object.>For declaring an object file which may be built by
    the project and the source files which have to be compiled in order to do
    this.

    <expand|item*|Archive.>For building an archive of object files.

    <expand|item*|Library.>For building a shared library.

    <expand|item*|Binary.>For building a binary from object files and
    archives or libraries.

    <expand|item*|Extern.>For dependencies of extern files.

    <expand|item*|Make.>Shell script for making an extern files if its
    dependencies are outdated.
  </description>

  An example of a project file is given below

  <\code>
    parent base

    prefix gn

    version 1.2.3

    \;

    source [name].gna

    \ \ \ \ \ \ \ \ header linalg.gnh calculus.gnh

    \;

    header appl.gnh

    \ \ \ \ \ \ \ \ header linalg.gnh calculus.gnh

    \;

    object [name].o

    \ \ \ \ \ \ \ \ source [name].gna

    \;

    archive appl.a

    \ \ \ \ \ \ \ \ object *.o
  </code>

  If this project is stored in <with|font family|tt|arithmetic.gnp>, the
  following might be a source file:

  <\code>
    module part1 in project arithmetic.gnp;

    <\with>
      font shape
    <|with>
      italic
    <|with>
      body
    </with>
  </code>

  <\remark>
    This section is very sketchy and the real format of project files has to
    be thought over more carefully. We merely wanted to illustrate the idea
    of project files and the attachment of source files to such projects.
  </remark>
</body>

<\initial>
  <\collection>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|reduction page bottom margin|15mm>
    <associate|page type|a4>
    <associate|reduction page left margin|25mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|disc-cv|1.3>
    <associate|cond-exports|4.5>
    <associate|rem-spec|1.2>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate>
      toc
    <|associate>
      <vspace*|1fn><with|font series|<quote|bold>|1
      Backup><htab|5mm>?<vspace|0.5fn>

      1.1 Multiple declaration parts in blocks?<htab|5mm>?

      1.2 Class specialization<htab|5mm>?

      1.3 Conditionally variable exports<htab|5mm>?
    </associate>
  </collection>
</auxiliary>
