
/******************************************************************************
* MODULE     : keyw_loop.cc
* DESCRIPTION: loops
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <single.hh>
#include <xtable.hh>
namespace mmx {

void
evaluator_rep::kw_loop_1 () {
  table loc;
  env= xtable (env, loc);
  loc (CONT_BREAK)= cons (env, trace, rem);

  cur= KW_LOOP_2;
  rem= cons (car(trace)[1], single (true), rem);
  kw_loop_2 ();
}

void
evaluator_rep::kw_loop_2 () { // pre-loop conditions
  // INPUT: (loop_2, cond_list, first_pass?, ...)
  list l= as_list (car(rem));
  while (!nil (l)) {
    generic tag= car (l);
    if (is_func (tag, MMX_WHILE, 1)) {
      cur= list (MMX_TRANSTYPE, tag[1], VAL_BOOL);
      rem= cons (KW_WHILE_1, cdr (l), cdr (rem));
      // OUTPUT: (cond, while_1, other_conds, first_pass?, ...)
      return;
    }
    else if (is_func (tag, MMX_FOR, 1) && as_bool (cadr (rem))) {
      cur= tag[1];
      rem= cons (KW_FOR_1, cdr (l), cdr (rem));
      // OUTPUT: (instr, for_1, other_conds, first_pass?, ...)
      return;
    }
    else if (is_func (tag, MMX_FORIN, 2)) {
      if (as_bool (cadr (rem))) {
	cur= KW_FORIN_1;
	// OUTPUT: (forin_1, cond_list, first_pass?, ...)
	kw_forin_1 ();
      }
      else {
	cur= KW_FORIN_4;
	// OUTPUT: (forin_4, cond_list, first_pass?, ...)
	kw_forin_4 ();
      }
      return;
    }
    l= cdr (l);
  }
  cur= car(trace)[2];
  rem= cons (KW_LOOP_3, cddr (rem));
  // OUTPUT: (loop_body, loop_3, ...)
}

void
evaluator_rep::kw_loop_3 () { // loop body
  // INPUT: (loop_3, loop_body_result, cont, ...)
  cur= KW_LOOP_4;
  rem= cons (car(trace)[1], cdr (rem));
  // OUTPUT: (loop_4, cond_list, cont, ...)
  kw_loop_4 ();
}  

void
evaluator_rep::kw_loop_4 () { // post-loop conditions
  // INPUT: (loop_4, cond_list, ...)
  list l= as_list (car (rem));
  while (!nil (l)) {
    generic tag= car (l);
    if (is_func (tag, MMX_UNTIL, 1)) {
      cur= list (MMX_TRANSTYPE, tag[1], VAL_BOOL);
      rem= cons (KW_UNTIL_1, cdr (l), cdr (rem));
      // OUTPUT: (cond, until_1, other_conds, ...)
      return;
    }
    else if (is_func (tag, MMX_STEP, 1)) {
      cur= tag[1];
      rem= cons (KW_STEP_1, cdr (l), cdr (rem));
      // OUTPUT: (instr, step_1, other_conds, first_pass?, ...)
      return;
    }
    l= cdr (l);
  }
  cur= KW_LOOP_2;
  rem= cons (car(trace)[1], single (false), cdr (rem));
  // OUTPUT: (loop_2, cond_list, false, ...)
  kw_loop_2 ();
}

} // namespace mmx
