
/******************************************************************************
* MODULE     : xtable.hh
* DESCRIPTION: extended tables (for local environments)
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __XTABLE_HH
#define __XTABLE_HH
#include <gentable.hh>
#include <table.hh>
namespace mmx {

class xtable;
class xtable_rep: public gentable_rep {
public:
  gentable base_t;
  gentable ext_t;

protected:
  int Type ();
  int Hash ();
  generic Copy ();

public:
  inline xtable_rep (gentable base_t2, gentable ext_t2):
    base_t (base_t2), ext_t (ext_t2) {}

  generic  get_binding (generic key);
  void     set_binding (generic binding);
  void     reset (generic key);
  list     entries ();
  bool     local_contains (generic key);
  generic  local_read (generic key);
  generic& local_write (generic key);

  friend xtable copy (xtable t);
};

class xtable {
protected:
  xtable_rep* rep;
  inline xtable (xtable_rep* rep2): rep (rep2) {}

public:
  inline xtable (gentable base_t): rep (new xtable_rep (base_t, table ())) {}
  inline xtable (gentable base_t, gentable ext_t):
    rep (new xtable_rep (base_t, ext_t)) {}
  inline friend xtable as_xtable (generic g) {
    if (gentype(g) != XTABLE_TYPE) type_mismatch (g, "xtable");
    return xtable ((xtable_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline operator gentable () { return gentable (rep); }
  inline xtable_rep* operator -> () { return rep; }

  inline generic  operator [] (generic g) { return rep->read (g); }
  inline generic& operator () (generic g) { return rep->write (g); }
  inline friend bool operator == (xtable a, xtable b) { return a->Equal (b); }
  inline friend bool operator != (xtable a, xtable b) { return !a->Equal (b); }
  inline friend ostream& operator << (ostream& out, xtable a) {
    a->Print (out); return out; }

  friend xtable copy (xtable a);

  friend class xtable_rep;
};

}
#endif // __XTABLE_HH
